/*
 * ***********************************************************************
 *
 * File 'tkCanvasPlusModes.h'.
 *
 * Tcl Interpreter for CanvasPlus widget.
 *
 * Written by Jorge Lima in Nov 23 14:44:03 WET 2004
 *
 */

#ifndef _TK_CANVAS_PLUS_INTERP_H
#define _TK_CANVAS_PLUS_INTERP_H

#include <tcl.h>
#include "CanvasPlusDefs.h"
#include "tkCanvasPlusDrawing.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct CP_RegisterInfo {
  char *procName;   /* procedure name to register */
  Tcl_ObjCmdProc *function;
  char *cfname;	    /* C function name for debug purposes.
		       Automaticaly filled by the CP_ENTRY macro. */
} CP_RegisterInfo;

/*
 * WARNING: the 'name' parameter must be a proper " delimited C string
 * constant, a pointer to a string won't work!
 */
#define CP_ENTRY(name,func) { name, func, #func "(...)" }
#define CP_LAST_ENTRY { NULL, NULL, NULL }


/* OPERATING MODES */

#define MODE_NAME_MAXLEN 24
typedef ClientData CP_AllocProc(CanvasPlus *);
typedef int CP_InitProc(CP_Mode *);
typedef void CP_FreeProc(ClientData);
typedef void CP_DrawProc(ClientData, CP_DrawOps *, CP_Area *);
typedef CP_Item *CP_ClosestProc(ClientData, CP_Point *, double);

/* Mode operations */
struct CP_ModeOps {
  CP_AllocProc *allocProc;
  CP_InitProc *initProc;
  CP_FreeProc *freeProc;
  CP_DrawProc *drawProc;
  CP_ClosestProc *closestProc;
};

/* Structure defining the mode */
struct CP_Mode {
  // Mode name
  char *name;
  // keep a handy copy of tkwin token
  Tk_Window tkwin;
  // Private interp related
  Tcl_Interp *interp;
  Tk_BindingTable bindingTable;
  // Item management fields
  Tcl_HashTable idTable;
  int nextId;
  CP_Item *firstItem;
  CP_Item *lastItem;
  // Painting case
  CP_PenSet pens;
  // Client specific token
  ClientData clientData;
  // Mode operations
  CP_ModeOps *ops;
};

/* Get currently available modes */
Tcl_Obj *CPListModes();

Tcl_Obj *CPGetUniqCmdName(Tcl_Interp *interp, char *base);

int CPSetMode(CanvasPlus *cPtr, Tcl_Obj *name, Tcl_Obj *mode);

void CPUnsetMode(CanvasPlus *cPtr);
/* Insert New Mode -- called when a module implementing a mode is loaded
   via the "load" command. */
int CPDefineMode(char *, CP_RegisterInfo *, CP_ModeOps *ops);


#ifdef __cplusplus
}
#endif

#endif /* _TK_CANVAS_PLUS_INTERP_H */
